#!/usr/bin/env python

# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!
#
#(c) QB89Dragon 2007/8 <hughescih@hotmail.com>
#(c) Whise 2008 <helderfraga@gmail.com>
#
# Settings Window

import sys
import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import gobject
import xml.dom.minidom
import os
import commands
import locale

# Fixes a broken images file if a bad theme has been selected
try:
	INSTALL_PREFIX = open("/etc/gnomenu/prefix").read()[:-1] 
except:
	INSTALL_PREFIX = '/usr'

def fixsettings(widget=None,event=None):
	os.system("cp " + INSTALL_PREFIX + "/lib/gnomenu/Settings_default.xml " + os.path.expanduser("~") + "/.GnoMenuSettings.xml")
	# Give it a second shot now
	try:
		import Globals
		global Globals
		#a = 9/0 # Force an error to occur for testing
	except:
		a = Message("Reverting to default settings seems to have failed to resolve the issue. Please run from a terminal 'GnoMenu.py run-in-window' and copy and paste the output to the gnome-look.org page where you downloaded the menu to help with debugging.",'GnoMenu', cancelfixsettings, cancelfixsettings, 1)

	gtk.main_quit()
		
def cancelfixsettings(widget=None,event=None):
	sys.exit()
try:
	import Globals
	#a = 9/0  # Force an error to occur for testing
except:
	from error_message import Message
	a = Message("The GnoMenu settings file has been corrupted and is unreadable, or you have selected a theme with broken dependencies. Revert to default settings?",'GnoMenu', fixsettings, cancelfixsettings)
	a.window.destroy()

# Import the desktop integration (gconf tweaking) module
from DesktopIntegration import DesktopIntegration

# Define autocategories
Categories = Globals.ThemeCategories
CategorySelection = ["","","",""]
CurrentTreeDataSet = "Menu"

DIModule = DesktopIntegration()

def destroy(event=None, anythingelse=None):
    # External Obliterator
    window.destroy()
    gtk.main_quit()

def Category_getvalue(type):
	for i in range(0,len(Categories)):
		if Categories[i] == type:
			return CategorySelection[i]

def Category_setvalue(type,value):
	global Category
	for i in range(0,len(Categories)):
		if Categories[i] == type:
			CategorySelection[i] = value

def LanguageChange(widget):
	global Language
	Dataset = widget.get_model()
	Index = widget.get_active()
	Row = Dataset[Index]
	Language = Row[1]
	LoadLanguage()
	
def TreeSelChange(widget):
	global CurrentTreeDataSet
	#Identify which category area was picked
	Dataset = widget.get_model()
	Index = widget.get_cursor()
	#Save old values from widget
	try:
		SaveTreeSelValue()
	except:pass

	CurrentTreeDataSet = Dataset[Index[0][0]][1]
	#Load new values to widget
	LoadStylePreview(CurrentTreeDataSet)
	

def DIvaluechanged(widget, index):
	Widget = wTree.get_widget("DITree")
	Dataset = Widget.get_model()
	Dataset[index][3] = not Dataset[index][3]
	if Dataset[index][3] == 1:
		DIModule.enable(index)
	else:
		DIModule.disable(index)
	Dataset[index][3] = DIModule.checkstate(index)
		
def SaveTreeSelValue(widget=None,event=None):
	# Saves the theme selection to memory
	Widget = wTree.get_widget("THThmSelCombo")
	Dataset = Widget.get_model()
	Index = Widget.get_active()
	
	Widget = wTree.get_widget("image1")
	try:
		Pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(Globals.ThemeDirectory  + CurrentTreeDataSet +"/" + Dataset[Index][1] + "/themepreview.png",200,390)
	except:
		Pixbuf = gtk.gdk.pixbuf_new_from_file(Globals.GraphicsDirectory + "theme.png")
	Widget.set_from_pixbuf(Pixbuf)
	XMLCredits = xml.dom.minidom.parse(Globals.ThemeDirectory  + CurrentTreeDataSet +"/" + Dataset[Index][1] + "/" + 'themedata.xml')
	XBase = XMLCredits.childNodes[0]
	# Load Language to all buttons, labels and other widgets
	# Set Labels
	SBase = XBase.getElementsByTagName("ContentData")
	try:
		Name = SBase[0].attributes["Name"].value
		Author = SBase[0].attributes["Author"].value
		Right = SBase[0].attributes["Copyright"].value
		Widget = wTree.get_widget("label1")
		Widget.set_text(Name + ' by: ' + Author + '\n\n' + Right)
	except:pass
	Pixbuf= None
	if Index >= 0:
		Category_setvalue(CurrentTreeDataSet,Dataset[Index][1])

def LoadStylePreview(type):
	# Load registered components of specific type

	Widget = wTree.get_widget("THThmSelCombo")
	Dataset = Widget.get_model()
	Dataset.clear()
	CurSel = Category_getvalue(type)
	SelIndex = -1
	x = 0
	themes = os.listdir(INSTALL_PREFIX + '/share/gnomenu/Themes/'+ type + '/')
	themes.sort()
	for i in themes: 
		Name = i
		Folder = i
		
		if Folder == CurSel:
			SelIndex = x
		try:
			Pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(Globals.ThemeDirectory + "/" + type +"/" + Folder + "/themepreview.png",64,64)
		except:
			Pixbuf = gtk.gdk.pixbuf_new_from_file(Globals.GraphicsDirectory + "theme.png")
		Dataset.append([Name,Folder,Pixbuf])
		x = x+1

	Widget.set_model(Dataset)
	Widget.set_active(SelIndex)	
		

def LoadLanguage():
	global Language
	# Disable unavailable languages
	loca = locale.getlocale(locale.LC_ALL)
	language = loca[0]
	if os.path.isfile(Globals.ProgramDirectory+"locale/Settings_LangPack_"+language+".xml") == 0:
		print "Selected Language " + Language + " is unavailable, reverting to default"
		Language = Default_Language


	# Load current language to widget values
	try:
		XMLLanguage = xml.dom.minidom.parse(Globals.ProgramDirectory+"locale/Settings_LangPack_"+language+".xml")
	except:
		XMLLanguage = xml.dom.minidom.parse(Globals.ProgramDirectory+"locale/Settings_LangPack_en.xml")
	XBase = XMLLanguage.childNodes[0]
	# Load Language to all buttons, labels and other widgets
	# Set Labels
	SBase = XBase.getElementsByTagName("Label")
	for node in SBase:	
		Label = node.attributes["Text"].value
		WidgetName = node.attributes["Widget"].value
		Widget = wTree.get_widget(WidgetName)
		if Widget and Label != '':
			Widget.set_text(Label)
		else:
			print "Recoverable Error: Widget not found: " + WidgetName
		
	SBase = XBase.getElementsByTagName("Expander")
	for node in SBase:	
		Label = node.attributes["Text"].value
		WidgetName = node.attributes["Widget"].value
		Widget = wTree.get_widget(WidgetName)
		if Widget:
			Widget.set_label(Label)
		else:
			print "Recoverable Error: Widget not found: " + WidgetName
		
	SBase = XBase.getElementsByTagName("Button")
	for node in SBase:	
		Label = node.attributes["Text"].value
		WidgetName = node.attributes["Widget"].value
		Widget = wTree.get_widget(WidgetName)
		if Widget:
			Widget.set_label(Label)
		else:
			print "Recoverable Error: Widget not found: " + WidgetName

	SBase = XBase.getElementsByTagName("Image")
	for node in SBase:	
		Filename = node.attributes["Image"].value
		WidgetName = node.attributes["Widget"].value
		Widget = wTree.get_widget(WidgetName)
		if Widget:
			Widget.set_from_file(Globals.GraphicsDirectory+Filename)
		else:
			print "Recoverable Error: Widget not found: " + WidgetName

	SBase = XBase.getElementsByTagName("ToolTip")
	for node in SBase:	
		Name = node.attributes["Text"].value
		WidgetName = node.attributes["Widget"].value
		Widget = wTree.get_widget(WidgetName)
		if Widget:
			Widget.set_tooltip_text(Name)
		else:
			print "Recoverable Error: Widget not found: " + WidgetName
	
	SBase = XBase.getElementsByTagName("Name")
	for node in SBase:	
		Name = node.attributes["Text"].value
		id = node.attributes["Id"].value
		loc = node.attributes["Loc"].value
		if loc == "Treeviewbase":
			Widget = wTree.get_widget("THTree")

			Dataset = Widget.get_model()
			for i in range(0,len(Dataset)):
				if Dataset[i][1] == id:
					Dataset[i][0] = Name
			Widget.set_model(Dataset)
		elif loc == "Treeviewcolumn":
			tree_column.set_title(Name)

		elif loc == "WindowTitle0":
			if not Globals.FirstUse:
				window.set_title(Name)
				# Fixme: Display welcome screen!
		elif loc == "WindowTitle1":
			if Globals.FirstUse:
				window.set_title(Name + " " + Globals.name + " " + Globals.version)
				# Fixme: Display welcome screen!
		else:
			print "Warning: Name location identifier not recognised from: " + node.toxml()

def show_question (message, title=''):
	md = gtk.MessageDialog(window, type=gtk.MESSAGE_QUESTION, 
			buttons=gtk.BUTTONS_YES_NO)
	md.set_title('?')
	md.set_markup(message)
	response = md.run()
	md.destroy()
	if response == gtk.RESPONSE_YES:
		return True
	return False

def show_message (message, title=''):

	md = gtk.MessageDialog(None, type=gtk.MESSAGE_INFO, 
			buttons=gtk.BUTTONS_OK)
	md.set_title(title)
	md.set_markup(message)
	md.run()
	md.destroy()

def show_uninstall_dialog (widget,event):
	Widget = wTree.get_widget("THThmSelCombo")
	Index =  Widget.get_active()
	Dataset = Widget.get_model()
	if show_question('Uninstall ' + str(CurrentTreeDataSet) + ' theme ' + str(Dataset[Index][1])) == True:
		install_prefix = 'gksu '
		print chr(34) + chr(39)+'rm -rf '+Globals.ThemeDirectory + CurrentTreeDataSet + "/" + Dataset[Index][1]+ chr(39)+chr(34)
		#Globals.ThemeDirectory  + CurrentTreeDataSet +"/" + Dataset[Index][1]
		os.system(install_prefix +chr(34) +'rm -rf '+ chr(39)+Globals.ThemeDirectory + CurrentTreeDataSet + "/" + Dataset[Index][1]+ chr(39)+chr(34))
	
def show_install_dialog (widget,event):
	"""Craete/Show the install-dialog."""
	# create filte
	flt = gtk.FileFilter()
	flt.add_pattern('*.tar.bz2')
	flt.add_pattern('*.tar.gz')
	flt.add_pattern('*.tar')
	# create dialog
	dlg = gtk.FileChooserDialog(buttons=(gtk.STOCK_CANCEL, 
			gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
	dlg.set_current_folder(os.environ['HOME'])
	dlg.set_title(('Install a new theme'))
	dlg.set_filter(flt)
	# run
	resp		= dlg.run()
	filename	= dlg.get_filename()
	dlg.destroy()
	if resp == gtk.RESPONSE_OK:
		# create new installer
		
		# try installing and show result dialog
		window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
		install (filename)	
		window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))
		

def install(filename):
	base	= os.path.basename(filename)
	ext		= str(filename)[len(str(filename)) -3:]
	# prepend "tar." if we have a bz2 or gz archive
	tar_opts = 'xfz'
	if ext == 'bz2':
		tar_opts = 'xfj'
	if not os.path.isdir('/tmp/gnomenu/'):
		os.system('mkdir ' + '/tmp/gnomenu/')
	tmpdir = '/tmp/gnomenu/'
	os.system('tar %s %s -C %s' % (tar_opts, chr(34)+filename+chr(34), tmpdir))
	for d in os.listdir(tmpdir) : #for each item in folders

		if os.path.exists(tmpdir + d) and os.path.isdir(tmpdir + d): 
			
			name = d

	XMLBase = xml.dom.minidom.parse('/tmp/gnomenu/'+ name+"/themedata.xml")
	#XBase = XMLBase.childNodes[0]
	try:
		SBase = XMLBase.getElementsByTagName("content")
		type = SBase[0].attributes["type"].value
	except:
		
		show_message('Error - theme not installed')
		os.system('rm -rf /tmp/gnomenu')
		return
	
	try:
		print type
	except:
		show_message('Error - theme not installed')
		os.system('rm -rf /tmp/gnomenu')
		return
	install_dir = INSTALL_PREFIX + '/share/gnomenu/Themes/' + type + '/'
	if is_installed(type + '/' + name):
		if show_question('Theme exists. Continue?') == False:
			os.system('rm -rf /tmp/gnomenu')
			return
	install_prefix = 'gksu '
	os.system(install_prefix +chr(34) +'tar '+tar_opts+' '+ chr(39)+ filename + chr(39)+ ' -C ' + chr(39) + install_dir + chr(39)+chr(34))
	if is_installed(type + '/' + name):
		show_message('Theme installed')
	else:
		show_message('Error - theme not installed')
	os.system('rm -rf /tmp/gnomenu')

def is_installed (name):
	return os.path.isdir(Globals.ThemeDirectory +  name)

def SaveSettings():
	#Language

	SBase = XBase.getElementsByTagName("Language")
	SBase[0].attributes["Language"].value = Language
	
	#Theme/Style Properties
	
	SBase = XBase.getElementsByTagName("Menu")
	SBase[0].attributes["Name"].value = CategorySelection[0]
	SBase = XBase.getElementsByTagName("Icon")
	SBase[0].attributes["Name"].value = CategorySelection[1]
	SBase = XBase.getElementsByTagName("Button")
	SBase[0].attributes["Name"].value = CategorySelection[2]
	SBase = XBase.getElementsByTagName("Widget")
	SBase[0].attributes["Name"].value = CategorySelection[3]

	#Advanced menu
	Widget = wTree.get_widget("ADWinKey")
	SBase = XBase.getElementsByTagName("SuperL")
	SBase[0].attributes["Enabled"].value = str(int(Widget.get_active()))
	Widget = wTree.get_widget("ShowIE")
	SBase = XBase.getElementsByTagName("ShowIE")
	SBase[0].attributes["Shownetandemail"].value = str(int(Widget.get_active()))	
	Widget = wTree.get_widget("GtkTheme")
	SBase = XBase.getElementsByTagName("GtkTheme")
	SBase[0].attributes["GtkColors"].value = str(int(Widget.get_active()))	
	
	
	file_object = open(Globals.HomeDirectory+"/.GnoMenuSettings.xml","w") 
	file_object.write(XMLSettings.toxml())  
	file_object.close()

def OK_Pressed(widget,event):
	SaveSettings()
	if show_question('Menu needs to restart , restart now?') == True:
		x = commands.getoutput("""ps axo "%p,%a" | grep "GnoMenu" | grep -v grep|cut -d',' -f1""")
		os.system('kill -9 ' + x)
	
	gtk.main_quit()
def CA_Pressed(widget,event):
	gtk.main_quit()
def RV_Pressed(widget,event):
	pass

# Force global welcome event if directed to by user
if len(sys.argv) == 2 and sys.argv[1] == "--welcome":
	Globals.FirstUse = True
	
# Set the Glade file
wTree = gtk.glade.XML(Globals.GladeSettingsFile)
window = wTree.get_widget("settingswindow")
#window.set_app_paintable(1)
#window.set_skip_taskbar_hint(1)
#window.set_skip_pager_hint(1)
window.set_keep_above(0)       #Make this always above other windows
	
# Load settings to memory
XMLSettings = xml.dom.minidom.parse(Globals.HomeDirectory+"/.GnoMenuSettings.xml")
XBase = XMLSettings.childNodes[0].childNodes[1]


# Load Language
SBase = XBase.getElementsByTagName("Language")
Language = SBase[0].attributes["Language"].value
Default_Language = SBase[0].attributes["Default"].value

# Load Theme Settings
for name in Categories:
	
	SBase = XBase.getElementsByTagName(name)
	print  SBase[0].attributes["Name"].value
	Category_setvalue(name, SBase[0].attributes["Name"].value)
	


#==================================
# Populate Theme selection treeview
#==================================
Widget = wTree.get_widget("THTree")

# Create internal listing of availability
tree_base = gtk.ListStore(str,str)
Widget.set_model(tree_base)

# Add the base selection categories
for i in range(0,len(Categories)):
	tree_base.append(["",Categories[i]])

# Create a single column
tree_column = gtk.TreeViewColumn("")
# Tell gtk what to render
celltext = gtk.CellRendererText()
tree_column.pack_start(celltext)
tree_column.add_attribute(celltext,'text',0)

Widget.append_column(tree_column)

Widget.set_cursor(0,tree_column)

Widget.connect("cursor-changed",TreeSelChange)

#==================================
# Configure theme selection list
#==================================

# Create list object
Widget = wTree.get_widget("THThmSelCombo")
theme_list = gtk.ListStore(str,str,gtk.gdk.Pixbuf)
Widget.set_model(theme_list)	

# Set how data should be displayed in the combobox
celltext = gtk.CellRendererText()
cellimage = gtk.CellRendererPixbuf()
Widget.pack_start(cellimage)
Widget.pack_start(celltext)
# Set so image is far left aligned
cellimage.set_property('xalign', 0)
celltext.set_property('xalign', 1)
Widget.add_attribute(celltext,'text',0)
Widget.add_attribute(cellimage, 'pixbuf',2)
Widget.set_active
# Set initial combobox values
LoadStylePreview(Categories[0])

Widget.connect("changed", SaveTreeSelValue)
SaveTreeSelValue(Widget)
#===================================
# Populate Desktop Integration Table
#===================================

#===================================
# Set Values to Advanced Menu
#===================================
Widget = wTree.get_widget("ADWinKey")
SBase = XBase.getElementsByTagName("SuperL")
KeyVal = SBase[0].attributes["Enabled"].value
Widget.set_active(int(KeyVal))

Widget = wTree.get_widget("ShowIE")
SBase = XBase.getElementsByTagName("ShowIE")
KeyVal = SBase[0].attributes["Shownetandemail"].value
Widget.set_active(int(KeyVal))

Widget = wTree.get_widget("GtkTheme")
SBase = XBase.getElementsByTagName("GtkTheme")
KeyVal = SBase[0].attributes["GtkColors"].value
Widget.set_active(int(KeyVal))

#==================================
# Connect main buttons
#==================================
Widget = wTree.get_widget("OK")
Widget.connect("button-press-event", OK_Pressed)
Widget = wTree.get_widget("CN")
Widget.connect("button-press-event", CA_Pressed)
Widget = wTree.get_widget("RV")
Widget.connect("button-press-event", RV_Pressed)
Widget = wTree.get_widget("THInst")
Widget.connect("button-press-event", show_install_dialog)
Widget = wTree.get_widget("THUnInst")
Widget.connect("button-press-event", show_uninstall_dialog)

#==================================
# Load language to widgets
#==================================
LoadLanguage()

def key_down (widget,event):
    # Key down
    key = event.hardware_keycode
    if key == 9:
		gtk.main_quit()
		
# Get the Main Window, and connect the "destroy" event
if (window):
    dic = {"on_MainMenu_destroy_event" : gtk.main_quit}
    wTree.signal_autoconnect(dic)
#window.connect("key-press-event", key_down)
window.connect("destroy", destroy)
window.connect("key-press-event", key_down)
window.present()
gtk.main()
        

